/*
    Copyright (C) 2009-2012 Cristi Dumitrescu <birt@eareview.net>
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#property copyright "birt"
#property link      "http://eareview.net"
#property show_inputs

extern string AutoShiftBackInfo = "Shift the time back 28 years to match the calendar.";
extern bool   AutoShiftBack = true;
extern string ManualShiftInfo1 = "Number of seconds to shift (negative or positive).";
extern string ManualShiftInfo2 = "Only works if AutoShiftBack is disabled.";
extern int    ManualShift = 0;

int      ExtPeriods[9] = { 1, 5, 15, 30, 60, 240, 1440, 10080, 43200 };
int      ExtPeriodCount = 9;

//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start()
  {
   if (AutoShiftBack) {
      ManualShift = -883612800;
   }
//----
   for (int i = 0; i < ExtPeriodCount; i++) {
      int HstHandle = FileOpenHistory(Symbol() + ExtPeriods[i] + ".hst", FILE_READ | FILE_BIN);
      int OutHandle = FileOpen(Symbol() + ExtPeriods[i] + ".hst", FILE_WRITE | FILE_BIN);
      if (!HstHandle) {
         Alert("Unable to open " + Symbol() + ExtPeriods[i] + ".hst. Skipping.");
         continue;
      }
      // read the header
      int dummy[15];
      int version = FileReadInteger(HstHandle, LONG_VALUE); // version
      string copy = FileReadString(HstHandle, 64); // copyright
      string sym = FileReadString(HstHandle, 12); // symbol
      int period = FileReadInteger(HstHandle, LONG_VALUE); // period
      int digits = FileReadInteger(HstHandle, LONG_VALUE); // digits
      FileReadArray(HstHandle, dummy, 0, 15);
      // write the header
      FileWriteInteger(OutHandle, version, LONG_VALUE);
      FileWriteString(OutHandle, copy, 64);
      FileWriteString(OutHandle, sym, 12);
      FileWriteInteger(OutHandle, period, LONG_VALUE);
      FileWriteInteger(OutHandle, digits, LONG_VALUE);
      FileWriteArray(OutHandle, dummy, 0, 15);
      while (!IsStopped()) {
         datetime cur_time = FileReadInteger(HstHandle, LONG_VALUE);
         if (FileIsEnding(HstHandle)) break;
         bool write_record = true;
         if (ManualShift > 0) {
            if (cur_time > 0x7FFFFFFF - ManualShift) {
               write_record = false;
            }
         }
         else {
            if (cur_time < -ManualShift) {
               write_record = false;
            }
         }
         double open = FileReadDouble(HstHandle, DOUBLE_VALUE);
         double low  = FileReadDouble(HstHandle, DOUBLE_VALUE);
         double high = FileReadDouble(HstHandle, DOUBLE_VALUE);
         double close = FileReadDouble(HstHandle, DOUBLE_VALUE);
         double volume = FileReadDouble(HstHandle, DOUBLE_VALUE);
         if (write_record) {
            cur_time += ManualShift;
            FileWriteInteger(OutHandle, cur_time, LONG_VALUE);
            FileWriteDouble(OutHandle, open, DOUBLE_VALUE);
            FileWriteDouble(OutHandle, low, DOUBLE_VALUE);
            FileWriteDouble(OutHandle, high, DOUBLE_VALUE);
            FileWriteDouble(OutHandle, close, DOUBLE_VALUE);
            FileWriteDouble(OutHandle, volume, DOUBLE_VALUE);
         }
         if (FileIsEnding(HstHandle)) break;
      }
      FileClose(HstHandle);
      FileClose(OutHandle);
      if (IsStopped()) break;
   }
   Alert("Processing complete.");
//----
   return(0);
  }
//+------------------------------------------------------------------+

